@@ -1,26 +1,17 @@
-/*************************************************************************
- *  Copyright (C) 2009, 2010 by Volker Lanz <vl@fidra.de>                *
- *  Copyright (C) 2016 by Teo Mrnjavac <teo@kde.org>                     *
- *  Copyright (C) 2017 by Andrius Štikonas <andrius@stikonas.eu>         *
- *                                                                       *
- *  This program is free software; you can redistribute it and/or        *
- *  modify it under the terms of the GNU General Public License as       *
- *  published by the Free Software Foundation; either version 3 of       *
- *  the License, or (at your option) any later version.                  *
- *                                                                       *
- *  This program is distributed in the hope that it will be useful,      *
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of       *
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
- *  GNU General Public License for more details.                         *
- *                                                                       *
- *  You should have received a copy of the GNU General Public License    *
- *  along with this program.  If not, see <http://www.gnu.org/licenses/>.*
- *************************************************************************/
+/*
+    SPDX-FileCopyrightText: 2017-2020 Andrius Štikonas <andrius@stikonas.eu>
+    SPDX-FileCopyrightText: 2019 Adriaan de Groot <groot@kde.org>
+
+    SPDX-License-Identifier: GPL-3.0-or-later
+*/
 
 #include "core/fstab.h"
 #include "util/externalcommand.h"
 #include "util/report.h"
 
+#include <algorithm>
+#include <array>
+
 #if defined(Q_OS_LINUX)
     #include <blkid/blkid.h>
 #endif
@@ -34,6 +25,8 @@
 
 static void parseFsSpec(const QString& m_fsSpec, FstabEntry::Type& m_entryType, QString& m_deviceNode);
 static QString findBlkIdDevice(const char *token, const QString& value);
+static void writeEntry(QTextStream& s, const FstabEntry& entry, std::array<unsigned int, 4> columnWidth);
+std::array<unsigned int, 4> fstabColumnWidth(const FstabEntryList& fstabEntries);
 
 struct FstabEntryPrivate
 {
@@ -59,6 +52,7 @@ FstabEntry::FstabEntry(const QString& fsSpec, const QString& mountPoint, const Q
     d->m_comment = comment;
 
     d->m_options = options.split(QLatin1Char(','));
+    d->m_options.removeAll(QStringLiteral("defaults"));
     parseFsSpec(d->m_fsSpec, d->m_entryType, d->m_deviceNode);
 }
 
@@ -78,7 +72,7 @@ FstabEntryList readFstabEntries( const QString& fstabPath )
             }
 
             QString comment = line.section( QLatin1Char('#'), 1 );
-            QStringList splitLine = line.section( QLatin1Char('#'), 0, 0 ).split( QRegularExpression(QStringLiteral("[\\s]+")), QString::SkipEmptyParts );
+            QStringList splitLine = line.section( QLatin1Char('#'), 0, 0 ).split( QRegularExpression(QStringLiteral("[\\s]+")), Qt::SkipEmptyParts );
 
             // We now split the standard components of /etc/fstab entry:
             // (0) path, or UUID, or LABEL, etc,
@@ -88,15 +82,20 @@ FstabEntryList readFstabEntries( const QString& fstabPath )
             // (4) dump frequency (optional, defaults to 0), no comment is allowed if omitted,
             // (5) pass number (optional, defaults to 0), no comment is allowed if omitted,
             // (#) comment (optional).
+            auto fsSpec = splitLine.at(0);
+            auto mountPoint = splitLine.at(1);
+            auto fsType = splitLine.at(2);
+            auto options = splitLine.at(3);
+
             switch (splitLine.length()) {
                 case 4:
-                    fstabEntries.push_back( {splitLine.at(0), splitLine.at(1), splitLine.at(2), splitLine.at(3) } );
+                    fstabEntries.push_back( {fsSpec, mountPoint, fsType, options } );
                     break;
                 case 5:
-                    fstabEntries.push_back( {splitLine.at(0), splitLine.at(1), splitLine.at(2), splitLine.at(3), splitLine.at(4).toInt() } );
+                    fstabEntries.push_back( {fsSpec, mountPoint, fsType, options, splitLine.at(4).toInt() } );
                     break;
                 case 6:
-                    fstabEntries.push_back( {splitLine.at(0), splitLine.at(1), splitLine.at(2), splitLine.at(3), splitLine.at(4).toInt(), splitLine.at(5).toInt(), comment.isEmpty() ? QString() : QLatin1Char('#') + comment } );
+                    fstabEntries.push_back( {fsSpec, mountPoint, fsType, options, splitLine.at(4).toInt(), splitLine.at(5).toInt(), comment.isEmpty() ? QString() : QLatin1Char('#') + comment } );
                     break;
                 default:
                     fstabEntries.push_back( { {}, {}, {}, {}, {}, {}, QLatin1Char('#') + line } );
@@ -142,6 +141,11 @@ const QStringList& FstabEntry::options() const
     return d->m_options;
 }
 
+const QString FstabEntry::optionsString() const
+{
+    return options().size() > 0 ? options().join(QLatin1Char(',')) : QStringLiteral("defaults");
+}
+
 int FstabEntry::dumpFreq() const
 {
     return d->m_dumpFreq;
@@ -203,6 +207,9 @@ static QString findBlkIdDevice(const char *token, const QString& value)
         rval = QString::fromLocal8Bit(c);
         free(c);
     }
+#else
+    Q_UNUSED(token);
+    Q_UNUSED(value);
 #endif
 
     return rval;
@@ -229,40 +236,50 @@ static void parseFsSpec(const QString& m_fsSpec, FstabEntry::Type& m_entryType,
     }
 }
 
-static void writeEntry(QTextStream& s, const FstabEntry& entry)
+
+// Used to nicely format fstab file
+std::array<unsigned int, 4> fstabColumnWidth(const FstabEntryList& fstabEntries)
+{
+    std::array<unsigned int, 4> columnWidth;
+
+#define FIELD_WIDTH(x) 3 + std::max_element(fstabEntries.begin(), fstabEntries.end(), [](const FstabEntry& a, const FstabEntry& b) {return a.x().length() < b.x().length(); })->x().length();
+
+    columnWidth[0] = FIELD_WIDTH(fsSpec);
+    columnWidth[1] = FIELD_WIDTH(mountPoint);
+    columnWidth[2] = FIELD_WIDTH(type);
+    columnWidth[3] = FIELD_WIDTH(optionsString);
+
+    return columnWidth;
+}
+
+static void writeEntry(QTextStream& s, const FstabEntry& entry, std::array<unsigned int, 4> columnWidth)
 {
     if (entry.entryType() == FstabEntry::Type::comment) {
         s << entry.comment() << "\n";
         return;
     }
 
-    QString options;
-    if (entry.options().size() > 0) {
-        options = entry.options().join(QLatin1Char(','));
-        if (options.isEmpty())
-            options = QStringLiteral("defaults");
-    }
-    else
-        options = QStringLiteral("defaults");
-
-    s << entry.fsSpec() << "\t"
-      << (entry.mountPoint().isEmpty() ? QStringLiteral("none") : entry.mountPoint()) << "\t"
-      << entry.type() << "\t"
-      << options << "\t"
-      << entry.dumpFreq() << "\t"
-      << entry.passNumber() << "\t"
+    s.setFieldAlignment(QTextStream::AlignLeft);
+    s.setFieldWidth(columnWidth[0]);
+    s << entry.fsSpec()
+      << qSetFieldWidth(columnWidth[1]) << (entry.mountPoint().isEmpty() ? QStringLiteral("none") : entry.mountPoint())
+      << qSetFieldWidth(columnWidth[2]) << entry.type()
+      << qSetFieldWidth(columnWidth[3]) << entry.optionsString() << qSetFieldWidth(0)
+      << entry.dumpFreq() << " "
+      << entry.passNumber() << " "
       << entry.comment() << "\n";
 }
 
 bool writeMountpoints(const FstabEntryList& fstabEntries, const QString& filename)
 {
-    Report report(nullptr);
-    QByteArray fstabContents;
+    QString fstabContents;
     QTextStream out(&fstabContents);
 
+    std::array<unsigned int, 4> columnWidth = fstabColumnWidth(fstabEntries);
+
     for (const auto &e : fstabEntries)
-        writeEntry(out, e);
+        writeEntry(out, e, columnWidth);
 
     ExternalCommand cmd;
-    return cmd.writeData(report, fstabContents, filename, 0);
+    return cmd.createFile(fstabContents.toLocal8Bit(), filename);
 }
