@@ -1,25 +1,19 @@
-/*************************************************************************
-  *  Copyright (C) 2017 by Andrius Štikonas <andrius@stikonas.eu>        *
- *                                                                       *
- *  This program is free software; you can redistribute it and/or        *
- *  modify it under the terms of the GNU General Public License as       *
- *  published by the Free Software Foundation; either version 3 of       *
- *  the License, or (at your option) any later version.                  *
- *                                                                       *
- *  This program is distributed in the hope that it will be useful,      *
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of       *
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
- *  GNU General Public License for more details.                         *
- *                                                                       *
- *  You should have received a copy of the GNU General Public License    *
- *  along with this program.  If not, see <http://www.gnu.org/licenses/>.*
- *************************************************************************/
+/*
+    SPDX-FileCopyrightText: 2017-2020 Andrius Štikonas <andrius@stikonas.eu>
+    SPDX-FileCopyrightText: 2018-2019 Caio Jordão Carvalho <caiojcarvalho@gmail.com>
+    SPDX-FileCopyrightText: 2019 Shubham Jangra <aryan100jangid@gmail.com>
+    SPDX-FileCopyrightText: 2020 Gaël PORTAY <gael.portay@collabora.com>
+    SPDX-FileCopyrightText: 2020 Adriaan de Groot <groot@kde.org>
+
+    SPDX-License-Identifier: GPL-3.0-or-later
+*/
 
 /** @file
 */
 
 #include "plugins/sfdisk/sfdiskbackend.h"
 #include "plugins/sfdisk/sfdiskdevice.h"
+#include "plugins/sfdisk/sfdiskgptattributes.h"
 
 #include "core/copysourcedevice.h"
 #include "core/copytargetbytearray.h"
@@ -125,6 +119,60 @@ QList<Device*> SfdiskBackend::scanDevices(const ScanFlags scanFlags)
     return result;
 }
 
+/*** @brief Fix up bogus JSON from `sfdisk --json /dev/sdb`
+ *
+ * The command `sfdisk --json /dev/sdb` outputs a JSON representation
+ * of the partition table, with general device characteristics and
+ * the list of partitions, **but**..
+ *
+ * This isn't necessarily valid JSON: in particular, when there are
+ * no partitions on the disk because it is empty / was recently zeroed /
+ * is a USB stick for testing purposes, the output is changed **only**
+ * by there being no partitions in the partition table. However,
+ * the comma (",") after sectorsize is still printed. Bogus output looks
+ * like this:
+ *
+ * {
+ *   "partitiontable": {
+ *       "label":"gpt",
+ *       "id":"1F9E80D9-DD78-024F-94A3-B61EC82B18C8",
+ *       "device":"/dev/sdb",
+ *       "unit":"sectors",
+ *       "firstlba":2048,
+ *       "lastlba":30949342,
+ *       "sectorsize":512,
+ *   }
+ * }
+ *
+ * That's not valid JSON because of the "," followed by nothing until
+ * the brace, and yields an empty object is passed to fromJson().
+ *
+ * We'll go through and check if there's a "," followed by whitespace
+ * and then a }. If there is, replace the ,.
+ *
+ * This is also fixed in util-linux 2.37.
+ */
+static void
+fixInvalidJsonFromSFDisk( QByteArray& s )
+{
+    // -1 if there is no comma (but then there's no useful JSON either),
+    //    not is 0 a valid place (the start) for a , in a JSON document.
+    int lastComma = s.lastIndexOf(',');
+    if ( lastComma > 0 )
+    {
+        for ( int charIndex = lastComma + 1; charIndex < s.length(); ++charIndex )
+        {
+            if ( s[charIndex] == '}' )
+            {
+                s[lastComma] = ' ';  // Erase that comma
+            }
+            if ( !isspace( s[charIndex] ) )
+            {
+                break;
+            }
+        }
+    }
+}
 
 /** Create a Device for the given device_node and scan it for partitions.
     @param deviceNode the device node (e.g. "/dev/sda")
@@ -139,11 +187,11 @@ Device* SfdiskBackend::scanDevice(const QString& deviceNode)
                           deviceNode });
     ExternalCommand sizeCommand(QStringLiteral("blockdev"), { QStringLiteral("--getsize64"), deviceNode });
     ExternalCommand sizeCommand2(QStringLiteral("blockdev"), { QStringLiteral("--getss"), deviceNode });
-    ExternalCommand jsonCommand(QStringLiteral("sfdisk"), { QStringLiteral("--json"), deviceNode }, QProcess::ProcessChannelMode::SeparateChannels );
+    ExternalCommand sfdiskJsonCommand(QStringLiteral("sfdisk"), { QStringLiteral("--json"), deviceNode }, QProcess::ProcessChannelMode::SeparateChannels );
 
     if ( sizeCommand.run(-1) && sizeCommand.exitCode() == 0
          && sizeCommand2.run(-1) && sizeCommand2.exitCode() == 0
-         && jsonCommand.run(-1) )
+         && sfdiskJsonCommand.run(-1) )
     {
         Device* d = nullptr;
         qint64 deviceSize = sizeCommand.output().trimmed().toLongLong();
@@ -203,10 +251,16 @@ Device* SfdiskBackend::scanDevice(const QString& deviceNode)
 
         if ( d )
         {
-            if (jsonCommand.exitCode() != 0)
-                return d;
+            if (sfdiskJsonCommand.exitCode() != 0) {
+                scanWholeDevicePartition(*d);
 
-            const QJsonObject jsonObject = QJsonDocument::fromJson(jsonCommand.rawOutput()).object();
+                return d;
+            }
+
+            auto s = sfdiskJsonCommand.rawOutput();
+            fixInvalidJsonFromSFDisk(s);
+
+            const QJsonObject jsonObject = QJsonDocument::fromJson(s).object();
             const QJsonObject partitionTable = jsonObject[QLatin1String("partitiontable")].toObject();
 
             if (!updateDevicePartitionTable(*d, partitionTable))
@@ -232,6 +286,28 @@ Device* SfdiskBackend::scanDevice(const QString& deviceNode)
     return nullptr;
 }
 
+/** Scans a Device for FileSystems spanning the whole block device
+
+    This method  will scan a Device for a FileSystem.
+    It tries to determine the FileSystem usage, reads the FileSystem label and creates
+    PartitionTable of type "none" and a single Partition object.
+*/
+void SfdiskBackend::scanWholeDevicePartition(Device& d) {
+    const QString partitionNode = d.deviceNode();
+    constexpr qint64 firstSector = 0;
+    const qint64 lastSector = d.totalLogical() - 1;
+    setPartitionTableForDevice(d, new PartitionTable(PartitionTable::TableType::none, firstSector, lastSector));
+    Partition *partition = scanPartition(d, partitionNode, firstSector, lastSector, QString(), false);
+
+    if (partition->fileSystem().type() == FileSystem::Type::Unknown) {
+        setPartitionTableForDevice(d, nullptr);
+        delete d.partitionTable();
+    }
+
+    if (!partition->roles().has(PartitionRole::Luks))
+        readSectorsUsed(d, *partition, partition->mountPoint());
+}
+
 /** Scans a Device for Partitions.
 
     This method  will scan a Device for all Partitions on it, detect the FileSystem for each Partition,
@@ -249,67 +325,13 @@ void SfdiskBackend::scanDevicePartitions(Device& d, const QJsonArray& jsonPartit
         const qint64 start = partitionObject[QLatin1String("start")].toVariant().toLongLong();
         const qint64 size = partitionObject[QLatin1String("size")].toVariant().toLongLong();
         const QString partitionType = partitionObject[QLatin1String("type")].toString();
-        PartitionTable::Flags activeFlags = partitionObject[QLatin1String("bootable")].toBool() ? PartitionTable::Flag::Boot : PartitionTable::Flag::None;
+        const bool bootable = partitionObject[QLatin1String("bootable")].toBool();
+        const auto lastSector = start + size - 1;
 
-        if (partitionType == QStringLiteral("C12A7328-F81F-11D2-BA4B-00A0C93EC93B"))
-            activeFlags |= PartitionTable::Flag::Boot;
-        else if (partitionType == QStringLiteral("21686148-6449-6E6F-744E-656564454649"))
-            activeFlags |= PartitionTable::Flag::BiosGrub;
+        Partition* part = scanPartition(d, partitionNode, start, lastSector, partitionType, bootable);
 
-        FileSystem::Type type = FileSystem::Type::Unknown;
-        type = detectFileSystem(partitionNode);
-        PartitionRole::Roles r = PartitionRole::Primary;
+        setupPartitionInfo(d, part, partitionObject);
 
-        if ( (d.partitionTable()->type() == PartitionTable::msdos || d.partitionTable()->type() == PartitionTable::msdos_sectorbased) &&
-            ( partitionType == QStringLiteral("5") || partitionType == QStringLiteral("f") ) ) {
-            r = PartitionRole::Extended;
-            type = FileSystem::Type::Extended;
-        }
-
-        // Find an extended partition this partition is in.
-        PartitionNode* parent = d.partitionTable()->findPartitionBySector(start, PartitionRole(PartitionRole::Extended));
-
-        // None found, so it's a primary in the device's partition table.
-        if (parent == nullptr)
-            parent = d.partitionTable();
-        else
-            r = PartitionRole::Logical;
-
-        FileSystem* fs = FileSystemFactory::create(type, start, start + size - 1, d.logicalSize());
-        fs->scan(partitionNode);
-
-        QString mountPoint;
-        bool mounted;
-        // sfdisk does not handle LUKS partitions
-        if (fs->type() == FileSystem::Type::Luks || fs->type() == FileSystem::Type::Luks2) {
-            r |= PartitionRole::Luks;
-            FS::luks* luksFs = static_cast<FS::luks*>(fs);
-            luksFs->initLUKS();
-            QString mapperNode = luksFs->mapperName();
-            mountPoint = FileSystem::detectMountPoint(fs, mapperNode);
-            mounted    = FileSystem::detectMountStatus(fs, mapperNode);
-        } else {
-            mountPoint = FileSystem::detectMountPoint(fs, partitionNode);
-            mounted = FileSystem::detectMountStatus(fs, partitionNode);
-        }
-
-        Partition* part = new Partition(parent, d, PartitionRole(r), fs, start, start + size - 1, partitionNode, availableFlags(d.partitionTable()->type()), mountPoint, mounted, activeFlags);
-
-        if (!part->roles().has(PartitionRole::Luks))
-            readSectorsUsed(d, *part, mountPoint);
-
-        if (fs->supportGetLabel() != FileSystem::cmdSupportNone)
-            fs->setLabel(fs->readLabel(part->deviceNode()));
-
-        if (d.partitionTable()->type() == PartitionTable::TableType::gpt) {
-            part->setLabel(partitionObject[QLatin1String("name")].toString());
-            part->setUUID(partitionObject[QLatin1String("uuid")].toString());
-        }
-
-        if (fs->supportGetUUID() != FileSystem::cmdSupportNone)
-            fs->setUUID(fs->readUUID(part->deviceNode()));
-
-        parent->append(part);
         partitions.append(part);
     }
 
@@ -318,10 +340,80 @@ void SfdiskBackend::scanDevicePartitions(Device& d, const QJsonArray& jsonPartit
     if (d.partitionTable()->isSectorBased(d))
         d.partitionTable()->setType(d, PartitionTable::msdos_sectorbased);
 
-    for (const Partition * part : qAsConst(partitions))
+    for (const Partition *part : qAsConst(partitions))
         PartitionAlignment::isAligned(d, *part);
 }
 
+Partition* SfdiskBackend::scanPartition(Device& d, const QString& partitionNode, const qint64 firstSector, const qint64 lastSector, const QString& partitionType, const bool bootable)
+{
+    PartitionTable::Flags activeFlags = bootable ? PartitionTable::Flag::Boot : PartitionTable::Flag::None;
+    if (partitionType == QStringLiteral("C12A7328-F81F-11D2-BA4B-00A0C93EC93B"))
+        activeFlags |= PartitionTable::Flag::Boot;
+    else if (partitionType == QStringLiteral("21686148-6449-6E6F-744E-656564454649"))
+        activeFlags |= PartitionTable::Flag::BiosGrub;
+
+    FileSystem::Type type = detectFileSystem(partitionNode);
+    PartitionRole::Roles r = PartitionRole::Primary;
+
+    if ( (d.partitionTable()->type() == PartitionTable::msdos || d.partitionTable()->type() == PartitionTable::msdos_sectorbased) &&
+        ( partitionType == QStringLiteral("5") || partitionType == QStringLiteral("f") ) ) {
+        r = PartitionRole::Extended;
+        type = FileSystem::Type::Extended;
+    }
+
+    // Find an extended partition this partition is in.
+    PartitionNode* parent = d.partitionTable()->findPartitionBySector(firstSector, PartitionRole(PartitionRole::Extended));
+
+    // None found, so it's a primary in the device's partition table.
+    if (parent == nullptr)
+        parent = d.partitionTable();
+    else
+        r = PartitionRole::Logical;
+
+    FileSystem* fs = FileSystemFactory::create(type, firstSector, lastSector, d.logicalSize());
+    fs->scan(partitionNode);
+
+    QString mountPoint;
+    bool mounted;
+    // sfdisk does not handle LUKS partitions
+    if (fs->type() == FileSystem::Type::Luks || fs->type() == FileSystem::Type::Luks2) {
+        r |= PartitionRole::Luks;
+        FS::luks* luksFs = static_cast<FS::luks*>(fs);
+        luksFs->initLUKS();
+        QString mapperNode = luksFs->mapperName();
+        mountPoint = FileSystem::detectMountPoint(fs, mapperNode);
+        mounted    = FileSystem::detectMountStatus(fs, mapperNode);
+    } else {
+        mountPoint = FileSystem::detectMountPoint(fs, partitionNode);
+        mounted = FileSystem::detectMountStatus(fs, partitionNode);
+    }
+
+    Partition* partition = new Partition(parent, d, PartitionRole(r), fs, firstSector, lastSector, partitionNode, availableFlags(d.partitionTable()->type()), mountPoint, mounted, activeFlags);
+
+    if (fs->supportGetLabel() != FileSystem::cmdSupportNone)
+        fs->setLabel(fs->readLabel(partition->deviceNode()));
+
+    if (fs->supportGetUUID() != FileSystem::cmdSupportNone)
+        fs->setUUID(fs->readUUID(partition->deviceNode()));
+
+    parent->append(partition);
+    return partition;
+}
+
+void SfdiskBackend::setupPartitionInfo(const Device &d, Partition *partition, const QJsonObject& partitionObject)
+{
+    if (!partition->roles().has(PartitionRole::Luks))
+        readSectorsUsed(d, *partition, partition->mountPoint());
+
+    if (d.partitionTable()->type() == PartitionTable::TableType::gpt) {
+        partition->setLabel(partitionObject[QLatin1String("name")].toString());
+        partition->setUUID(partitionObject[QLatin1String("uuid")].toString());
+        partition->setType(partitionObject[QLatin1String("type")].toString());
+        QString attrs = partitionObject[QLatin1String("attrs")].toString();
+        partition->setAttributes(SfdiskGptAttributes::toULongLong(attrs.split(QLatin1Char(' '))));
+    }
+}
+
 bool SfdiskBackend::updateDevicePartitionTable(Device &d, const QJsonObject &jsonPartitionTable)
 {
     QString tableType = jsonPartitionTable[QLatin1String("label")].toString();
@@ -405,66 +497,95 @@ FileSystem::Type SfdiskBackend::detectFileSystem(const QString& partitionPath)
                                  QStringLiteral("--query=property"),
                                  partitionPath });
 
-    if (udevCommand.run(-1) && udevCommand.exitCode() == 0) {
-        QRegularExpression re(QStringLiteral("ID_FS_TYPE=(\\w+)"));
-        QRegularExpression re2(QStringLiteral("ID_FS_VERSION=(\\w+)"));
-        QRegularExpressionMatch reFileSystemType = re.match(udevCommand.output());
-        QRegularExpressionMatch reFileSystemVersion = re2.match(udevCommand.output());
+    QString typeRegExp = QStringLiteral("ID_FS_TYPE=(\\w+)");
+    QString versionRegExp = QStringLiteral("ID_FS_VERSION=(\\w+)");
+
+    QString name = {};
+
+    rval = runDetectFileSystemCommand(udevCommand, typeRegExp, versionRegExp, name);
+
+    // Fallback to blkid which has slightly worse detection but it works on whole block device filesystems.
+    if (rval == FileSystem::Type::Unknown) {
+        ExternalCommand blkidCommand(QStringLiteral("blkid"), { partitionPath });
+        typeRegExp = QStringLiteral("TYPE=\"(\\w+)\"");
+        versionRegExp = QStringLiteral("SEC_TYPE=\"(\\w+)\"");
+        rval = runDetectFileSystemCommand(blkidCommand, typeRegExp, versionRegExp, name);
+    }
+
+    if (rval == FileSystem::Type::Unknown) {
+        qWarning() << "unknown file system type " << name << " on " << partitionPath;
+    }
+    return rval;
+}
+
+FileSystem::Type SfdiskBackend::runDetectFileSystemCommand(ExternalCommand& command, QString& typeRegExp, QString& versionRegExp, QString& name)
+{
+    FileSystem::Type rval = FileSystem::Type::Unknown;
+
+    if (command.run(-1) && command.exitCode() == 0) {
+        QRegularExpression re(typeRegExp);
+        QRegularExpression re2(versionRegExp);
+        QRegularExpressionMatch reFileSystemType = re.match(command.output());
+        QRegularExpressionMatch reFileSystemVersion = re2.match(command.output());
 
-        QString s;
         if (reFileSystemType.hasMatch()) {
-            s = reFileSystemType.captured(1);
+            name = reFileSystemType.captured(1);
         }
 
-        QString version;
+        QString version = {};
         if (reFileSystemVersion.hasMatch()) {
             version = reFileSystemVersion.captured(1);
         }
-
-        if (s == QStringLiteral("ext2")) rval = FileSystem::Type::Ext2;
-        else if (s == QStringLiteral("ext3")) rval = FileSystem::Type::Ext3;
-        else if (s.startsWith(QStringLiteral("ext4"))) rval = FileSystem::Type::Ext4;
-        else if (s == QStringLiteral("swap")) rval = FileSystem::Type::LinuxSwap;
-        else if (s == QStringLiteral("ntfs")) rval = FileSystem::Type::Ntfs;
-        else if (s == QStringLiteral("reiserfs")) rval = FileSystem::Type::ReiserFS;
-        else if (s == QStringLiteral("reiser4")) rval = FileSystem::Type::Reiser4;
-        else if (s == QStringLiteral("xfs")) rval = FileSystem::Type::Xfs;
-        else if (s == QStringLiteral("jfs")) rval = FileSystem::Type::Jfs;
-        else if (s == QStringLiteral("hfs")) rval = FileSystem::Type::Hfs;
-        else if (s == QStringLiteral("hfsplus")) rval = FileSystem::Type::HfsPlus;
-        else if (s == QStringLiteral("ufs")) rval = FileSystem::Type::Ufs;
-        else if (s == QStringLiteral("vfat")) {
-            if (version == QStringLiteral("FAT32"))
-                rval = FileSystem::Type::Fat32;
-            else if (version == QStringLiteral("FAT16"))
-                rval = FileSystem::Type::Fat16;
-            else if (version == QStringLiteral("FAT12"))
-                rval = FileSystem::Type::Fat12;
-        }
-        else if (s == QStringLiteral("btrfs")) rval = FileSystem::Type::Btrfs;
-        else if (s == QStringLiteral("ocfs2")) rval = FileSystem::Type::Ocfs2;
-        else if (s == QStringLiteral("zfs_member")) rval = FileSystem::Type::Zfs;
-        else if (s == QStringLiteral("hpfs")) rval = FileSystem::Type::Hpfs;
-        else if (s == QStringLiteral("crypto_LUKS")) {
-            if (version == QStringLiteral("1"))
-                rval = FileSystem::Type::Luks;
-            else if (version == QStringLiteral("2")) {
-                rval = FileSystem::Type::Luks2;
-            }
-        }
-        else if (s == QStringLiteral("exfat")) rval = FileSystem::Type::Exfat;
-        else if (s == QStringLiteral("nilfs2")) rval = FileSystem::Type::Nilfs2;
-        else if (s == QStringLiteral("LVM2_member")) rval = FileSystem::Type::Lvm2_PV;
-        else if (s == QStringLiteral("f2fs")) rval = FileSystem::Type::F2fs;
-        else if (s == QStringLiteral("udf")) rval = FileSystem::Type::Udf;
-        else if (s == QStringLiteral("iso9660")) rval = FileSystem::Type::Iso9660;
-        else if (s == QStringLiteral("linux_raid_member")) rval = FileSystem::Type::LinuxRaidMember;
-        else if (s == QStringLiteral("BitLocker")) rval = FileSystem::Type::BitLocker;
-        else if (s == QStringLiteral("apfs")) rval = FileSystem::Type::Apfs;
-        else if (s == QStringLiteral("minix")) rval = FileSystem::Type::Minix;
-        else
-            qWarning() << "unknown file system type " << s << " on " << partitionPath;
+        rval = fileSystemNameToType(name, version);
     }
+    return rval;
+}
+
+FileSystem::Type SfdiskBackend::fileSystemNameToType(const QString& name, const QString& version)
+{
+    FileSystem::Type rval = FileSystem::Type::Unknown;
+
+    if (name == QStringLiteral("ext2")) rval = FileSystem::Type::Ext2;
+    else if (name == QStringLiteral("ext3")) rval = FileSystem::Type::Ext3;
+    else if (name.startsWith(QStringLiteral("ext4"))) rval = FileSystem::Type::Ext4;
+    else if (name == QStringLiteral("swap")) rval = FileSystem::Type::LinuxSwap;
+    else if (name == QStringLiteral("ntfs")) rval = FileSystem::Type::Ntfs;
+    else if (name == QStringLiteral("reiserfs")) rval = FileSystem::Type::ReiserFS;
+    else if (name == QStringLiteral("reiser4")) rval = FileSystem::Type::Reiser4;
+    else if (name == QStringLiteral("xfs")) rval = FileSystem::Type::Xfs;
+    else if (name == QStringLiteral("jfs")) rval = FileSystem::Type::Jfs;
+    else if (name == QStringLiteral("hfs")) rval = FileSystem::Type::Hfs;
+    else if (name == QStringLiteral("hfsplus")) rval = FileSystem::Type::HfsPlus;
+    else if (name == QStringLiteral("ufs")) rval = FileSystem::Type::Ufs;
+    else if (name == QStringLiteral("vfat")) {
+        if (version == QStringLiteral("FAT32"))
+            rval = FileSystem::Type::Fat32;
+        else if (version == QStringLiteral("FAT16") || version == QStringLiteral("msdos")) // blkid uses msdos for both FAT16 and FAT12
+            rval = FileSystem::Type::Fat16;
+        else if (version == QStringLiteral("FAT12"))
+            rval = FileSystem::Type::Fat12;
+    }
+    else if (name == QStringLiteral("btrfs")) rval = FileSystem::Type::Btrfs;
+    else if (name == QStringLiteral("ocfs2")) rval = FileSystem::Type::Ocfs2;
+    else if (name == QStringLiteral("zfs_member")) rval = FileSystem::Type::Zfs;
+    else if (name == QStringLiteral("hpfs")) rval = FileSystem::Type::Hpfs;
+    else if (name == QStringLiteral("crypto_LUKS")) {
+        if (version == QStringLiteral("1"))
+            rval = FileSystem::Type::Luks;
+        else if (version == QStringLiteral("2")) {
+            rval = FileSystem::Type::Luks2;
+        }
+    }
+    else if (name == QStringLiteral("exfat")) rval = FileSystem::Type::Exfat;
+    else if (name == QStringLiteral("nilfs2")) rval = FileSystem::Type::Nilfs2;
+    else if (name == QStringLiteral("LVM2_member")) rval = FileSystem::Type::Lvm2_PV;
+    else if (name == QStringLiteral("f2fs")) rval = FileSystem::Type::F2fs;
+    else if (name == QStringLiteral("udf")) rval = FileSystem::Type::Udf;
+    else if (name == QStringLiteral("iso9660")) rval = FileSystem::Type::Iso9660;
+    else if (name == QStringLiteral("linux_raid_member")) rval = FileSystem::Type::LinuxRaidMember;
+    else if (name == QStringLiteral("BitLocker")) rval = FileSystem::Type::BitLocker;
+    else if (name == QStringLiteral("apfs")) rval = FileSystem::Type::Apfs;
+    else if (name == QStringLiteral("minix")) rval = FileSystem::Type::Minix;
 
     return rval;
 }
